@@ -46,7 +46,7 @@ pub fn preprocess_ns_def(
               Err(e) => return Err(e),
             }
           } else {
-            Calcit::Thunk(Box::new(resolved_code))
+            Calcit::Thunk(Box::new(resolved_code), None)
           };
           // println!("\nwriting value to: {}/{} {:?}", ns, def, v);
           program::write_evaled_def(ns, def, v.clone())?;
@@ -251,7 +251,7 @@ fn process_list_call(
       }
     }
     (Calcit::Syntax(name, name_ns), _) => match name.as_str() {
-      "quasiquote" | "quote-replace" => Ok((
+      "quasiquote" => Ok((
         preprocess_quasiquote(&name, &name_ns, args, scope_defs, file_ns, program_code)?,
         None,
       )),
